/***************************************************************************
 *   Copyright (C) 2005 by Aleš Zamuda   *
 *   ales.zamuda@uni-mb.si   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "chesswidgetimpl.h"
#include <qtextedit.h>
#include "geneticsimulation.h"
#include "transactionstartevent.h"
#include <string>
#include <iostream>
#include <qstring.h>
#include <qregexp.h>
#include <qlineedit.h>
#include <vector>
#include "gray.h"
#include "populacija.h"
#include "communicationthread.h"
#include <qspinbox.h>
#include <kled.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include "evaluationparameters.h"
#include <qlcdnumber.h>
#include "schemecalculator.h"
#include "osebek.h"
#include <qlabel.h>
#include <kprogress.h>
#include <qdatetime.h>
#include "selectionbaker.h"
#include "selectionlinear.h"
#include "selectionnolinear.h"
#include "selectionproportional.h"
#include "selectiontournament.h"
#include <knuminput.h>
#include "transactionstartevent.h"

//TODO: odstrani odvečne #include

ChessWidgetImpl::ChessWidgetImpl(QWidget *parent, const char *name)
		:ChessWidget(parent, name) {
	geneticSimulation = new genetic::GeneticSimulation();
	geneticSimulation->setTargetWidget(this);
	geneticSimulation->start();
}

void ChessWidgetImpl::customEvent(QCustomEvent *event) {
	if ((int)event->type() == gui::TransactionStartEvent::TransactionStart) {
		gui::TransactionStartEvent *startEvent =
		    (gui::TransactionStartEvent *)event;
		std::string msg = startEvent->message;
		if (msg=="refreshNrEvals") {
			refreshNrEvals();
		} else
		if (msg=="refreshStats") {
			refreshAfterNewGeneration();
		} else
		if (msg=="finish") {
			simulationStop();
		} else {
			writeToTextArea(msg);
		}
	} else {
		ChessWidgetImpl::customEvent(event);
	}
}

void ChessWidgetImpl::writeToTextArea(std::string &line) {
	teSimulationOutput->insert(line);
	/*QString("%1: finished generation #%2.\n")
			.arg(QDateTime::currentDateTime().toString("d.M@h:m"))
	.arg(geneticSimulation->populacija->getStGeneracije()));*/
}

void ChessWidgetImpl::simulationStart() {
	btnSimulationStart->setEnabled(false);
	btnSimulationStop->setEnabled(true);
	btnPause->setEnabled(true);
	sbPopSize->setEnabled(false);
	sbCodingBitsEvalPieceKnight->setEnabled(false);
	sbCodingBitsEvalPieceBishop->setEnabled(false);
	sbCodingBitsEvalKingRookPositions->setEnabled(false);
	sbCodingBitsEvalPiecesPositions->setEnabled(false);
	sbCodingBitsEvalPieceQueen->setEnabled(false);
	sbCodingBitsEvalAttacksCount->setEnabled(false);
	sbCodingBitsEvalKingHasCastled->setEnabled(false);
	sbCodingBitsEvalKingShieldedAttacks->setEnabled(false);
	sbCodingBitsEvalKingEdgePositions->setEnabled(false);
	sbCodingBitsEvalDoubledPawns->setEnabled(false);
	sbCodingBitsEvalPieceRook->setEnabled(false);
	sbCodingBitsEvalStoppedPawns->setEnabled(false);
	geneticSimulation->isMultiselection=false;
	std::string msg("start");
	geneticSimulation->addTransaction(msg);
	kLedSimulationRunning->on();
}

void ChessWidgetImpl::simulationStop() {
	geneticSimulation->populacija->ustaviSimulacijo=!geneticSimulation->populacija->ustaviSimulacijo;

	btnSimulationStart->setEnabled(true);
	btnSimulationStop->setEnabled(false);
	btnPause->setEnabled(false);
	sbPopSize->setEnabled(true);
	sbCodingBitsEvalPieceKnight->setEnabled(true);
	sbCodingBitsEvalPieceBishop->setEnabled(true);
	sbCodingBitsEvalKingRookPositions->setEnabled(true);
	sbCodingBitsEvalPiecesPositions->setEnabled(true);
	sbCodingBitsEvalPieceQueen->setEnabled(true);
	sbCodingBitsEvalAttacksCount->setEnabled(true);
	sbCodingBitsEvalKingHasCastled->setEnabled(true);
	sbCodingBitsEvalKingShieldedAttacks->setEnabled(true);
	sbCodingBitsEvalKingEdgePositions->setEnabled(true);
	sbCodingBitsEvalDoubledPawns->setEnabled(true);
	sbCodingBitsEvalPieceRook->setEnabled(true);
	sbCodingBitsEvalStoppedPawns->setEnabled(true);
	kLedSimulationRunning->off();

	rbSelectionProportional->setEnabled(true);
	rbSelectionBakerSimpleRanking->setEnabled(true);
	rbSelectionLinearRanking->setEnabled(true);
	rbSelectionNolinearRanking->setEnabled(true);
	rbSelectionTournament->setEnabled(true);
}

void ChessWidgetImpl::simulationPause() {
	geneticSimulation->populacija->simulacijaNaPavzi=!geneticSimulation->populacija->simulacijaNaPavzi;
	btnPause->toggle();
}

void ChessWidgetImpl::changedSimulationG(int G) {
	geneticSimulation->populacija->setG(G);
}

void ChessWidgetImpl::changedSimulationPopSize(int pop_size) {
	geneticSimulation->populacija->setPop_size(pop_size);
	sbE->setMaxValue(pop_size);
	sbTournamentSize->setMaxValue(pop_size);
	//selekcijski pritisk pri linearni selekciji je odvisen od pop_size, spremenimo.
	kdniSelectionPressure->setMinValue(1.0/pop_size);
	kdniSelectionPressure->setMaxValue(2.0/pop_size);
}

void ChessWidgetImpl::changedSimulationM(int m) {
	geneticSimulation->populacija->setM(m);
}

void ChessWidgetImpl::changedSimulationP_m(double p_m) {
	geneticSimulation->populacija->setP_m(p_m);
}

void ChessWidgetImpl::changedSimulationP_k(double p_k) {
	geneticSimulation->populacija->setP_k(p_k);
}

void ChessWidgetImpl::changedSimulationE(int e) {
	geneticSimulation->populacija->setE(e);
}

void ChessWidgetImpl::changedSimulationEvaluationTime(int evalTime) {
	geneticSimulation->engine->setMoveTimeDuration(evalTime);
}

void ChessWidgetImpl::changedSimulationAnswerSpeedPrize(int kolicnik) {
	geneticSimulation->engine->setKolicnikCasa(kolicnik);
}

/**
 * To se lahko zgodi le, ko je simulacija ustavljena:
 * -znova inicializiramo populacijo
 */
void ChessWidgetImpl::nrBitsChanged() {
	geneticSimulation->evaluationParameters->bitResolutions[0]=
	    QString(sbCodingBitsEvalPieceKnight->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[1]=
	    QString(sbCodingBitsEvalPieceBishop->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[2]=
	    QString(sbCodingBitsEvalPieceRook->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[3]=
	    QString(sbCodingBitsEvalPieceQueen->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[4]=
	    QString(sbCodingBitsEvalAttacksCount->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[5]=
	    QString(sbCodingBitsEvalPiecesPositions->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[6]=
	    QString(sbCodingBitsEvalDoubledPawns->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[7]=
	    QString(sbCodingBitsEvalStoppedPawns->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[8]=
	    QString(sbCodingBitsEvalKingRookPositions->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[9]=
	    QString(sbCodingBitsEvalKingEdgePositions->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[10]=
	    QString(sbCodingBitsEvalKingShieldedAttacks->text()).toInt();
	geneticSimulation->evaluationParameters->bitResolutions[11]=
	    QString(sbCodingBitsEvalKingHasCastled->text()).toInt();
}

void ChessWidgetImpl::selectionTypeChanged() {
	using namespace genetic;
	Populacija::SELECTION sel = Populacija::SELECTION_PROPORTIONAL;
	if (rbSelectionProportional->isChecked()) {} else if (rbSelectionBakerSimpleRanking->isChecked()) {
		sel = Populacija::SELECTION_BAKER;
	} else if (rbSelectionLinearRanking->isChecked()) {
		sel = Populacija::SELECTION_LINEAR;
	} else if (rbSelectionNolinearRanking->isChecked()) {
		sel = Populacija::SELECTION_NOLINEAR;
	} else if (rbSelectionTournament->isChecked()) {
		sel = Populacija::SELECTION_TOURNAMENT;
	}

	geneticSimulation->populacija->changeSelectionType(sel);
}

/**
 * Ponovno napolnimo vektor za shemo in osvežimo rezultate izračunov.
 * Če je katera shema napačno vnešena, jo obravnavamo kot same zvezdice.
 */
void ChessWidgetImpl::schemeChanged() {
	vectShema.clear();

	QLineEdit* schemeBitLists[12]={
	                                  leSchemeEvalPieceKnight,
	                                  leSchemeEvalPieceBishop,
	                                  leSchemeEvalPieceRook,
	                                  leSchemeEvalPieceQueen,

	                                  leSchemeEvalAttacksCount,
	                                  leSchemeEvalPiecesPositions,
	                                  leSchemeEvalDoubledPawns,
	                                  leSchemeEvalStoppedPawns,
	                                  leSchemeEvalKingRookPositions,
	                                  leSchemeEvalKingEdgePositions,
	                                  leSchemeEvalKingShieldedAttacks,
	                                  leSchemeEvalKingHasCastled};

	for (int j = 0; j < 12; j++) {
		QRegExp regExp(QString("^[01*]{%1}$").arg(geneticSimulation->evaluationParameters->bitResolutions[j]));
		//vektor ne ustreza regEXP, naredimo default
		if (!regExp.exactMatch(schemeBitLists[j]->text())) {
			for (int pos = 0; pos < geneticSimulation->evaluationParameters->bitResolutions[j]; pos++)
				vectShema.push_back(-1);//dont care, če so napake!
			continue;
		}

		//prekopiramo ta listEdit v vektor za shemo
		for (int i = 0; i < geneticSimulation->evaluationParameters->bitResolutions[j]; i++) {
			int val=-1;
			QCharRef c = schemeBitLists[j]->text()[i];
			if (c=='0') val=0;
			if (c=='1') val=1;
			vectShema.push_back(val);//dont care, če so napake!
		}
	}

	assert(!geneticSimulation->populacija->getStariOsebki() ||
	       vectShema.size()==(*geneticSimulation->populacija->getStariOsebki())[0]->biti.size());

	refreshSchemeStatistics();
}

void ChessWidgetImpl::gray2DecChanged(const QString&) {
	QRegExp regExp(QString("^[01]{%1}$").arg(leGray2Dec->text().length()));
	if (regExp.exactMatch(leGray2Dec->text())) {
		//najprej text2gray!
		std::vector<int> binCode,grayCode;
		for (int i = 0; i < (int)leGray2Dec->text().length(); i++)
			grayCode.push_back(leGray2Dec->text()[i]=='0'?0:1);

		genetic::GrayCode gc;
		gc.grayToBin(grayCode, binCode);
		int dec=0;
		for (int i = 0; i < (int)binCode.size(); i++)
			if (binCode[i]==1) dec+=1<<(binCode.size()-1-i);
		leGray2DecResult->setText(QString("%1").arg(dec));
	}
}

void ChessWidgetImpl::dec2GrayChanged(const QString&) {
	QRegExp regExp(QString("^[0-9]{%1}$").arg(leDec2Gray->text().length()));
	if (regExp.exactMatch(leDec2Gray->text())) {
		//najprej dec2bin!
		int dec = leDec2Gray->text().toInt();
		std::vector<int> binCode,grayCode;
		for (int i = 30; i >= 0; i--) {
			int b = 1 << i;
			if (dec >= b) {
				dec-=b;
				binCode.push_back(1);
			} else if (!binCode.empty())
				binCode.push_back(0);//brez trailingZeros
		}
		genetic::GrayCode gc;
		gc.binToGray(binCode, grayCode);
		QString str;
		for (int i = 0; i < (int)grayCode.size(); i++)
			str+=grayCode[i]==0?"0":"1";
		leDec2GrayResult->setText(str);
	}
}

void ChessWidgetImpl::refreshSchemeStatistics() {
	//shemo lahko izračunamo šele PO prvi generaciji
	if (geneticSimulation->populacija->getStariOsebki()==NULL) return;

	genetic::SchemeCalculator calc(vectShema,
	                               *geneticSimulation->populacija->getStariOsebki());
	calc.izracunaj(geneticSimulation->populacija->getLastF(),
	               geneticSimulation->populacija->getPop_size(),
	               geneticSimulation->populacija->getP_k(),
	               geneticSimulation->populacija->getM(),
	               geneticSimulation->populacija->getP_m());

	lcdSchemeF_t->display(QString("%1").arg(calc.getFPovprecni()));
	lcdSchemeEvalS0t->display(QString("%1").arg(calc.getEvalS0t()));
	lcdSchemeDeltaS0->display(QString("%1").arg(calc.getDolzinaSheme()));
	lcdSchemeOs0->display(QString("%1").arg(calc.getRedSheme()));
	lcdSchemeEs0t->display(QString("%1").arg(calc.getStPripadajocihS0()));
	lcdSchemeQs0->display(QString("%1").arg(calc.getKolicnikSheme()));
	lcdSchemeEs0t1->display(QString("%1").arg(calc.getNapovedZaShemoS0t1()));
}

void ChessWidgetImpl::refreshSimulationStatistics() {
	lcdStatsGenerationNumber->display(QString("%1").arg(geneticSimulation->populacija->getStGeneracije()));
	
	lcdStatsEvaluaitonSum->display(QString("%1").arg(geneticSimulation->populacija->getLastF()));
	lcdStatsEvaluationBest->display(QString("%1").arg(geneticSimulation->populacija->getOcenaNajboljsegaVPopulaciji()));
	kProgressSimmulationProgress->setValue(geneticSimulation->populacija->getStGeneracije());

	teSimulationOutput->insert(QString("%1: finished generation #%2.\n")
			.arg(QDateTime::currentDateTime().toString("d.M@h:m"))
			.arg(geneticSimulation->populacija->getStGeneracije()));
	
	refreshNrEvals();
}

void ChessWidgetImpl::refreshNrEvals() {
	lcdStatsEvaluationNumber->display(QString("%1").arg(geneticSimulation->populacija->getStOcenitevDoSedaj()));

	double evalsTotal=
			geneticSimulation->populacija->getG()*
			geneticSimulation->populacija->getP_k()*
			(1+geneticSimulation->populacija->getP_m())
			*50.0;
	
	double secF = (evalsTotal-geneticSimulation->populacija->getStOcenitevDoSedaj())*
			(geneticSimulation->engine->getMoveTimeDuration()+20.0)/1000.0;
	int sec=(int)secF;
	int min = sec/60;
	sec -= min*60;
	lblTimeLeft->setText(QString("%1m %2s").arg(min).arg(sec));
}

void ChessWidgetImpl::refreshBestGeneStatistics() {
	if (geneticSimulation->populacija->getStariOsebki()==NULL) return;

	std::vector<bool> &bestBits = (*geneticSimulation->populacija->getStariOsebki())[0]->biti;
	genetic::chessengine::EvaluationParameters ep;
	ep.fromBits(bestBits);

	lblBestGenesEvalPieceKnight->setText(QString("%1").arg(ep.evalPieceKnight));
	lblBestGenesEvalPieceBishop->setText(QString("%1").arg(ep.evalPieceBishop));
	lblBestGenesEvalPieceRook->setText(QString("%1").arg(ep.evalPieceRook));
	lblBestGenesEvalPieceQueen->setText(QString("%1").arg(ep.evalPieceQueen));

	lblBestGenesEvalDoubledPawns->setText(QString("%1").arg(ep.evalDoubledPawns));
	lblBestGenesEvalStoppedPawns->setText(QString("%1").arg(ep.evalStoppedPawns));
	lblBestGenesEvalPiecesPositions->setText(QString("%1").arg(ep.evalPiecesPosition));
	lblBestGenesEvalAttacksCount->setText(QString("%1").arg(ep.evalAttacksCount));
	lblBestGenesEvalKingEdgePositions->setText(QString("%1").arg(ep.evalKingEdgePositions));
	lblBestGenesEvalKingRookPositions->setText(QString("%1").arg(ep.evalKingRookPositions));
	lblBestGenesEvalKingShieldedAttacks->setText(QString("%1").arg(ep.evalKingHasShieldedAttacks));
	lblBestGenesEvalKingHasCastled->setText(QString("%1").arg(ep.evalKingHasCastled));
}

void ChessWidgetImpl::refreshAfterNewGeneration() {
	refreshSimulationStatistics();
	refreshBestGeneStatistics();
	refreshSchemeStatistics();
}

void ChessWidgetImpl::changedTournamentSize(int k) {
	geneticSimulation->populacija->selectionTournament->k=k;
}

void ChessWidgetImpl::changeSelectionPressure(double q) {
	geneticSimulation->populacija->selectionLinear->q=q;
}

void ChessWidgetImpl::changeSelectionPressureNolinear(double q) {
	geneticSimulation->populacija->selectionNolinear->q=q;
}

void ChessWidgetImpl::changeMaximalEvaluation(double max) {
	geneticSimulation->populacija->selectionBaker->MAX = max;
}

void ChessWidgetImpl::simulationMultiselection() {
	rbSelectionProportional->setEnabled(false);
	rbSelectionBakerSimpleRanking->setEnabled(false);
	rbSelectionLinearRanking->setEnabled(false);
	rbSelectionNolinearRanking->setEnabled(false);
	rbSelectionTournament->setEnabled(false);

	geneticSimulation->isMultiselection=true;
	
	btnSimulationStart->setEnabled(false);
	btnSimulationStop->setEnabled(true);
	btnPause->setEnabled(true);
	sbPopSize->setEnabled(false);
	sbCodingBitsEvalPieceKnight->setEnabled(false);
	sbCodingBitsEvalPieceBishop->setEnabled(false);
	sbCodingBitsEvalKingRookPositions->setEnabled(false);
	sbCodingBitsEvalPiecesPositions->setEnabled(false);
	sbCodingBitsEvalPieceQueen->setEnabled(false);
	sbCodingBitsEvalAttacksCount->setEnabled(false);
	sbCodingBitsEvalKingHasCastled->setEnabled(false);
	sbCodingBitsEvalKingShieldedAttacks->setEnabled(false);
	sbCodingBitsEvalKingEdgePositions->setEnabled(false);
	sbCodingBitsEvalDoubledPawns->setEnabled(false);
	sbCodingBitsEvalPieceRook->setEnabled(false);
	sbCodingBitsEvalStoppedPawns->setEnabled(false);
	std::string msg("start");
	geneticSimulation->addTransaction(msg);
	kLedSimulationRunning->on();
}


void ChessWidgetImpl::loadFile(const QString &fileName) {
	std::string str=fileName;
	geneticSimulation->loadFile(str);
}

void ChessWidgetImpl::saveFile(const QString &fileName) {
	std::string str=fileName;
	geneticSimulation->saveFile(str);
}

#include "chesswidgetimpl.moc"
