/***************************************************************************
 *   Copyright (C) 2005 by Aleš Zamuda   *
 *   ales.zamuda@uni-mb.si   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "communicationthread.h"
#include <sstream>
#include <iostream>
#include <fstream>
#include <qdatetime.h>
#include <unistd.h>
#include "geneticsimulation.h"
using namespace std;

namespace genetic {
	namespace chessengine {
		EngineCommunication::EngineCommunication() {
			engineFileName="../../engine/qchess";
			//engineFileName="../../engine/AZChess.exe";
			moveTimeDuration=5000;
			kolicnikCasa=0.1f;
			log.open("engine.log");
		}

		EngineCommunication::~EngineCommunication() {
			std::cout << "quit"<<std::endl;
		}

		void EngineCommunication::setMoveTimeDuration(int moveTimeDuration) {
			this->moveTimeDuration = moveTimeDuration;
		}
		
		void EngineCommunication::setKolicnikCasa(float kolicnikCasa) {
			this->kolicnikCasa = kolicnikCasa;
		}

		int EngineCommunication::getMoveTimeDuration() {
			return moveTimeDuration;
		}
		
#define SMER_NOTRI 0
#define SMER_VEN 1
		/**
		* Če smo uspeli zagnati motor, trenutno nit napravimo v demonsko,
		* da komunicira z njim.
		 */
		int EngineCommunication::startEngineProcess() {
			//pripravimo pipi
			int pServer[2];
			int pClient[2];
			pipe(pServer);
			pipe(pClient);
			//podvojimo proces
			int pid;
			pid = fork();

			if (pid < 0) return -1;
			if(pid == 0) {//child: spawn-amo engine
				close(0);
				close(1);
				close(2);//err naj ne sporoča.
				dup(pClient[SMER_NOTRI]);//input=output iz clienta
				dup(pServer[SMER_VEN]);//output=output od serverja
				
				execlp(engineFileName.c_str(),engineFileName.c_str(),NULL);//ne sme return
				log<<"Napaka pri zagonu engina!"<<std::endl;
				return -1;
			} else {//client
				close(0);
				close(1);
				dup(pServer[SMER_NOTRI]);
				dup(pClient[SMER_VEN]);
				std::cout<<"uci\nucinewgame"<<std::endl;
			}
			return 0;
		}

		float EngineCommunication::oceniGene(const string &geni) {
			float ocena=0;
			//podamo gene
			std::cout<<"setoption name evaluationGenes " << geni << std::endl;
			//log <<"setoption name evaluationGenes " << geni << std::endl;
				
			std::string line;
			std::string bestmove("bestmove");
			for (int indexPoteze=0; indexPoteze < (int)fens.size(); indexPoteze++) {
				std::cout << fens[indexPoteze] << "\ngo movetime "<< moveTimeDuration << std::endl;
				//log << fens[indexPoteze] << "\ngo movetime "<< moveTimeDuration << std::endl;
				QTime moveTime;
				moveTime.start();
				//std::cerr << fens[indexPoteze] << "\ngo movetime 300" << std::endl;
				while(getline(cin,line)) {
					//log << line<<std::endl;
					//std::cerr<<line<<std::endl;
					if (line.substr(0,8)==bestmove) {
						int timeDeltaMiliSek=moveTime.elapsed();
						string poteza=line.substr(9,(int)line.size());
						//std::cerr<<poteza<<"!="<<results[indexPoteze]<<std::endl;
						if (poteza==results[indexPoteze]) {
							std::cerr<<indexPoteze+1;
							std::stringstream a;
							a<<indexPoteze+1;
							geneticSimulation->posljiEvalMsgVGui(a.str());
							ocena+=1.0f;
							//upoštevamo še normaliziran čas
							float cas=(float)timeDeltaMiliSek/moveTimeDuration;
							if(cas<1) {
								cas=1-cas;//jemali bomo, kako hitro smo rešili.
								ocena+=cas*kolicnikCasa;
							}
						} else {
							std::cerr<<".";
							geneticSimulation->posljiEvalMsgVGui(std::string("."));
						}
						//std::cerr <<"@"<<indexPoteze<< " ocena="<<ocena<<"\t";
						usleep(500000);
						break;
					}
				}
			}
			geneticSimulation->posljiEvalMsgVGui(std::string("\n"));
			geneticSimulation->posljiEvalMsgVGui(std::string("refreshNrEvals"));
			
			return ocena;
		}
	}
}
