/***************************************************************************
 *   Copyright (C) 2005 by Aleš Zamuda   *
 *   ales.zamuda@uni-mb.si   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "evaluationparameters.h"
#include <sstream>
#include "gray.h"
#include <iostream>

namespace genetic {

	namespace chessengine {

		EvaluationParameters::EvaluationParameters(QObject *parent, const char *name)
			: QObject(parent, name) {
			evalPiecePawn=100;//nedotaknjeno!
			evalPieceKnight = 250;//+-256
			evalPieceBishop = 300;//+-256
			evalPieceRook = 500;//+-256
			evalPieceQueen = 900;//+-256

			evalAttacksCount=30/*1*/;//0..127->(/40)*0..3.2
			evalPiecesPosition=0/*1*/;//0..127->(/40)*0..3,2
			evalDoubledPawns = 50;//0..255
			evalStoppedPawns = 30;//0..255

			evalKingRookPositions = 80;//0..255
			evalKingEdgePositions = 30;//0..255
			evalKingHasShieldedAttacks = 0/*5*/;//0..255
			evalKingHasCastled=50;//0..255

			bitResolutions[0]=9;
			bitResolutions[1]=9;
			bitResolutions[2]=9;
			bitResolutions[3]=9;
			bitResolutions[4]=8;
			bitResolutions[5]=8;
			bitResolutions[6]=9;
			bitResolutions[7]=9;
			bitResolutions[8]=9;
			bitResolutions[9]=9;
			bitResolutions[10]=9;
			bitResolutions[11]=9;
		}


		EvaluationParameters::~EvaluationParameters() {}

		int EvaluationParameters::getNrBitsNeededForGenes() {
			int bits=0;
			for (int i = 0; i < (int)(sizeof(bitResolutions)/sizeof(*bitResolutions)); i++)
				bits+=bitResolutions[i];
			return bits;//106; 10*512, 2*256
		}

		void EvaluationParameters::getDefaultChromosome(std::vector<bool> &defaultChromosome) {
			toBits(defaultChromosome);
			assert((int)defaultChromosome.size()==getNrBitsNeededForGenes());
		}

		std::string EvaluationParameters::serializeParameters() {
			std::stringstream s;
			s << evalPiecePawn
			<< " " << evalPieceKnight
			<< " " << evalPieceBishop
			<< " " << evalPieceRook
			<< " " << evalPieceQueen

			<< " " << evalAttacksCount
			<< " " << evalPiecesPosition
			<< " " << evalDoubledPawns
			<< " " << evalStoppedPawns

			<< " " << evalKingRookPositions
			<< " " << evalKingEdgePositions
			<< " " << evalKingHasShieldedAttacks
			<< " " << evalKingHasCastled;
			std::string ss = s.str();
			return ss;
		}

		//kar bomo dali notri, bo kasneje kodirano po grayu
		void EvaluationParameters::toBits(std::vector<bool> &dst) {
			//  LEGENDA:
			//  evalPiecePawn=100;//nedotaknjeno!
			pushBits(evalPieceKnight, 256, bitResolutions[0], dst);//8 bits
			pushBits(evalPieceBishop, 300, bitResolutions[1], dst);
			pushBits(evalPieceRook, 500, bitResolutions[2], dst);
			pushBits(evalPieceQueen, 900, bitResolutions[3], dst);

			pushBits(evalAttacksCount, 128, bitResolutions[4], dst);
			pushBits(evalPiecesPosition, 128, bitResolutions[5], dst);
			pushBits(evalDoubledPawns, 256, bitResolutions[6], dst);
			pushBits(evalStoppedPawns, 256, bitResolutions[7], dst);

			pushBits(evalKingRookPositions, 256, bitResolutions[8], dst);
			pushBits(evalKingEdgePositions, 256, bitResolutions[9], dst);
			pushBits(evalKingHasShieldedAttacks, 256, bitResolutions[10], dst);
			pushBits(evalKingHasCastled, 256, bitResolutions[11], dst);
		}

		//predhodno dekodirano iz graya
		void EvaluationParameters::fromBits(const std::vector<bool> &src) {
			int pos=0;
			getInt(evalPieceKnight, 256, bitResolutions[0],pos, src);
			pos+=bitResolutions[0];
			getInt(evalPieceBishop, 300, bitResolutions[1],pos, src);
			pos+=bitResolutions[1];
			getInt(evalPieceRook, 500, bitResolutions[2],pos, src);
			pos+=bitResolutions[2];
			getInt(evalPieceQueen, 900, bitResolutions[3],pos, src);
			pos+=bitResolutions[3];

			getInt(evalAttacksCount, 128, bitResolutions[4],pos, src);
			pos+=bitResolutions[4];
			getInt(evalPiecesPosition, 128, bitResolutions[5],pos, src);
			pos+=bitResolutions[5];
			getInt(evalDoubledPawns, 256, bitResolutions[6],pos, src);
			pos+=bitResolutions[6];
			getInt(evalStoppedPawns, 256, bitResolutions[7],pos, src);
			pos+=bitResolutions[7];

			getInt(evalKingRookPositions, 256, bitResolutions[8],pos, src);
			pos+=bitResolutions[8];
			getInt(evalKingEdgePositions, 256, bitResolutions[9],pos, src);
			pos+=bitResolutions[9];
			getInt(evalKingHasShieldedAttacks, 256, bitResolutions[10],pos, src);
			pos+=bitResolutions[10];
			getInt(evalKingHasCastled, 256, bitResolutions[11],pos, src);
			pos+=bitResolutions[11];
			assert(pos==getNrBitsNeededForGenes());
		}

		void EvaluationParameters::pushBits(int val, int mid, int nrBits, std::vector<bool> &dst) {
			//val=1, resol=5bits ->00001
			//val=5, resol=3bits ->
			int resolution =  1 << nrBits;
			val = convertForPush(val, mid, resolution);

			if (!(val>=0 && val < 1<<nrBits))
				std::cerr<<val<<std::endl;
			assert(val>=0 && val < 1<<nrBits);

			//pretvorimo v grayev kod: MSB prej notri (ima manjši index)
			std::vector<int> tmpVectBin;
			for (int i = nrBits-1; i >= 0; i--) {
				int b = 1 << i;
				tmpVectBin.push_back(val>=b);
				if (val>=b) val-=b;
			}

			//zakodiramo v Gray
			std::vector<int> tmpVectGrey;
			GrayCode gc;
			gc.binToGray(tmpVectBin, tmpVectGrey);
			for (int i = 0; i < (int)tmpVectGrey.size(); i++)
				dst.push_back(tmpVectGrey[i]);
		}

		void EvaluationParameters::getInt(int &val, int mid, int nrBits, int index, const std::vector<bool> &src) {
			val=0;
			std::vector<int> tmpVectGrey;

			//skopiramo v vmesni vektor za grayevo pretvorbo
			for (int i = 0; i<nrBits; i++)
				tmpVectGrey.push_back(src[i+index]);//MSB nižji index
			//odkodiramo iz Gray
			std::vector<int> tmpVectBin;
			GrayCode gc;
			gc.grayToBin(tmpVectGrey, tmpVectBin);

			for (int i = 0; i < nrBits; i++) {
				if (tmpVectBin[i]!=0)
					val|=1<<(nrBits-i-1);//MSB nižji index
			}
			
			int resolution =  1 << nrBits;
			val=convertForPop(val, mid, resolution);
			assert(val>=0 && val >= mid-resolution/2 && val <= mid+resolution);
		}

		int EvaluationParameters::convertForPush(int val, int mid, int resolution) {
			int potisniVrednostDesno=mid-resolution/2;
			if (potisniVrednostDesno>0)
				val -= potisniVrednostDesno;//potisnemo vrednost levo

			if (val >= resolution) val=resolution-1;//odrežemo veliko resolucijo zgoraj
			if (val < 0) val=0;//odrežemo preveliko resolucijo od spodaj

			return val;
		}

		int EvaluationParameters::convertForPop(int val, int mid, int resolution) {
			//vrednost potisnemo na desno
			int potisniVrednostDesno=mid-resolution/2;
			if (potisniVrednostDesno>0)
				val += potisniVrednostDesno;
	
			return val;
		}

		void EvaluationParameters::readChromosome(std::ifstream &in) {
			in >> evalPiecePawn
				>> evalPieceKnight
				>> evalPieceBishop
				>> evalPieceRook
				>> evalPieceQueen
				
				>> evalAttacksCount
				>> evalPiecesPosition
				>> evalDoubledPawns
				>> evalStoppedPawns
				
				>> evalKingRookPositions
				>> evalKingEdgePositions
				>> evalKingHasShieldedAttacks
				>> evalKingHasCastled;
		}
	}

}
