/***************************************************************************
 *   Copyright (C) 2005 by Aleš Zamuda   *
 *   ales.zamuda@uni-mb.si   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GENETIC_CHESSENGINEEVALUATIONPARAMETERS_H
#define GENETIC_CHESSENGINEEVALUATIONPARAMETERS_H

#include <qobject.h>
#include <vector>
#include <string>
#include <fstream>

namespace genetic {

	namespace chessengine {

		/**
		@author Aleš Zamuda
		*/
		class EvaluationParameters : public QObject {
		public:
			EvaluationParameters(QObject *parent = 0, const char *name = 0);
			~EvaluationParameters();
			void toBits(std::vector<bool> &dst);
			void fromBits(const std::vector<bool> &src);
			int getNrBitsNeededForGenes();
			void getDefaultChromosome(std::vector<bool> &defaultChromosome);
			std::string serializeParameters();
			void readChromosome(std::ifstream &in);

		public:
			int evalPiecePawn;
			int evalPieceKnight;
			int evalPieceBishop;
			int evalPieceRook;
			int evalPieceQueen;

			int evalAttacksCount;
			int evalPiecesPosition;
			int evalDoubledPawns;
			int evalStoppedPawns;

			int evalKingRookPositions;
			int evalKingEdgePositions;
			int evalKingHasShieldedAttacks;
			int evalKingHasCastled;

			int bitResolutions[12];

		private:
			void pushBits(int val, int mid, int resolution, std::vector<bool> &dst);
			void getInt(int &val, int mid, int resolution, int index, const std::vector<bool> &src);
			int convertForPop(int val, int mid, int resolution);
			int convertForPush(int val, int mid, int resolution);
		};

	}

}

#endif
