/***************************************************************************
 *   Copyright (C) 2005 by Aleš Zamuda   *
 *   ales.zamuda@uni-mb.si   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GENETICGENETICSIMULATION_H
#define GENETICGENETICSIMULATION_H

#include <qevent.h>
#include <string>
#include <vector>

namespace genetic {
	namespace chessengine {
		class EvaluationParameters;
		class EngineCommunication;
	}
	class Populacija;
	class Osebek;
	/**
	@author Aleš Zamuda
	*/
	class GeneticSimulation {
	public:
		GeneticSimulation();
		~GeneticSimulation();

	public:
		void start();

		void addTransaction(const char *command);
		void setTargetWidget(QWidget* target);
		void addTransaction(const std::string &s);

		float GeneticSimulation::evaluate(Osebek *osebek);
		void posljiEvalMsgVGui(const std::string &s);
		void loadFile(const std::string &fileName);
		void saveFile(const std::string &fileName);

		std::string bestMessage;
		Populacija *populacija;
		chessengine::EngineCommunication *engine;
		chessengine::EvaluationParameters *evaluationParameters;
		bool isMultiselection;
		
	private:
		void GeneticSimulation::serveTransactions();
		void doInitPopulation();
		void saveBestMessage();
		void readFensResults();
		void readFens();
		void posljiRefreshMsgVGui();
		void posljiFinishMsgVGui();

		static void* threadExecute(void *X);
		void run();

		bool isContinue;
		std::list<std::string> transactions;
		QWidget *targetWidget;
	};

}
#endif
