#include "osebek.h"

#include <cstdlib>
#include <cmath>
#include <algorithm>
#include <iostream>
#include "gray.h"
#include <cassert>
#include "populacija.h"
#include "geneticsimulation.h"
#include "evaluationparameters.h"

namespace genetic {
//"""Inicializira osebek in shrani povezavo v populacijo."""
// Osebek::Osebek(int m, Populacija &populacija, int id) : m(m), populacija(&populacija), ocena(-1), id(id) {
// }

Osebek::Osebek() {
	populacija=0;
	m=-1;
	id=-1;
	ocena=-1;
}

//"""Genotip osebka napolni z naključnimi biti."""
void Osebek::nakljucnoNapolni() {
	biti.clear();
	for (int i=0; i < m; i++) biti.push_back(rand()%2);
}
	
//"""Izpiše podatke o osebku."""
void Osebek::izpis() {
	//std::cerr << ocena;
	assert(m>0 && id>0);
	if ((int)biti.size()!=m) {
		std::cerr << "bitov="<<(int)biti.size()<<std::endl;
		std::cerr << "potrebno="<<m<<std::endl;
		
	}
	assert((int)biti.size()==m);
	for (int i = 0; i < m; i++) 
		std::cerr << biti[i];

	populacija->geneticSimulation->evaluationParameters->fromBits(biti);
	
	std::cerr 
			<< " (" << populacija->geneticSimulation->evaluationParameters->serializeParameters()<<")"
			<< " $$=" << ocena
			<< " [#"<<id<<"]"<<std::endl;
}

//"""Po pravilih mutacije mutira bite v genotipu osebka po določeni verjetnosti."""
void Osebek::mutiraj(float p_m) {
	assert(m>0 && id>0);
	assert((int)biti.size()==m);
	assert(populacija);

	for (int i=0; i < m; i++) {
		if ((float)rand()/RAND_MAX < p_m) {
			biti[i]=!biti[i];
			ocena=-1;//resetiramo oceno, tega bomo ponovno ocenili!
			if (populacija->izpisujMutacije)
				std::cerr << " Mutiral osebek " << id
					<<"(@"<<i<<") ";
		}
	}
}


//"""Izvede ocenitev tega osebka."""
void Osebek::oceni() {
	assert(m>0 && id>0);

	int sz=(int)biti.size();
	assert(sz==m);
	//ocena=oceniPoSheflerju();
	assert(populacija->geneticSimulation);
	ocena=populacija->geneticSimulation->evaluate(this);
}
/*
int oceniPoSheflerju() {
	static std::vector<int> g1, g2;
	g1.resize(m/2);
	g2.resize(m/2);

	copy(biti.begin(), 		biti.begin()+m/2,	g1.begin());
	copy(biti.begin()+m/2, 	biti.end(),			g2.begin());

	static std::vector<int> b1, b2;
	b1.resize(m/2);
	b2.resize(m/2);

	static GrayCode gray;
	gray.grayToBin(g1, b1);
	gray.grayToBin(g2, b2);

	double x1=intVal(b1);
	x1=real(x1);
	double x2=intVal(b2);
	x2=real(x2);

	double stevec=
		-x1*sin(sqrt(x1>=0?x1:-x1))
		-x2*sin(sqrt(x2>=0?x2:-x2));
	double C=418.982887;
	ocena=stevec/2+C;
}

double Osebek::intVal(const std::vector<int>& binCode) {
	assert(m>0 && id>0);
	assert((int)binCode.size()==m/2);

	double val=0;
	for (int i = 0; i < m/2; i++) {
		val=val*2+binCode[i];
	}

	return val;
}

double Osebek::real(double x) {
	return -500+x*1000/((1<<24)-1);
}*/
}
