#include "populacija.h"
#include <cstdlib>
#include <iostream>
#include "geneticsimulation.h"
#include "osebek.h"
#include <vector>
#include <unistd.h>
#include "selectionbaker.h"
#include "selectionlinear.h"
#include "selectionnolinear.h"
#include "selectionproportional.h"
#include "selectiontournament.h"

namespace genetic {
	//"""Pripravi začetne parametre simulacije in inicializira naključno začetno populacijo."""
	Populacija::Populacija() {
		G=200;
		pop_size=30;
		p_k=0.75;
		p_m=0.01;
		m=106;
		e=2;
		osebki=0;
		osebkiStaraGeneracija=0;
		ustaviSimulacijo=false;
		simulacijaNaPavzi=false;
		trenutnaSelekcija=SELECTION_TOURNAMENT;
		menjajSelekcijoVNaslednjiGeneraciji=false;

		//ustvarimo selekcije in ponudimo kazalec na populacijo
		selectionBaker = new selection::SelectionBaker(this);
		selectionLinear = new selection::SelectionLinear(this);
		selectionNolinear = new selection::SelectionNolinear(this);
		selectionProportional = new selection::SelectionProportional(this);
		selectionTournament = new selection::SelectionTournament(this);
		
		izbranaSelekcija=selectionTournament;
		initLog();
	}

	bool Populacija::setG(int G) {
		if (G >= stGeneracije) {
			this->G=G;
			return true;
		}
		return false;
	}

	bool Populacija::setPop_size(int pop_size) {
		if (osebki==0) {
			this->pop_size = pop_size;
			return true;
		}
		return false;
	}

	void Populacija::setP_k(float p_k) {
		this->p_k = p_k;
	}

	void Populacija::setP_m(float p_m) {
		this->p_m = p_m;
	}

	void Populacija::setM(int m) {
		this->m = m;
	}

	void Populacija::setE(int e) {
		this->e = e;
	}

	int Populacija::getG() {
		return G;
	}
	
	int Populacija::getPop_size() {
		return pop_size;
	}
	
	float Populacija::getP_k() {
		return p_k;
	}
	
	float Populacija::getP_m() {
		return p_m;
	}
	
	int Populacija::getM() {
		return m;
	}

	int Populacija::getE() {
		return e;
	}
	
	int Populacija::getStGeneracije() {
		return this->stGeneracije;
	}
	
	float Populacija::getLastF() {
		return this->lastF;
	}
			
	float Populacija::getOcenaNajboljsegaVPopulaciji() {
		return this->ocenaNajboljsegaVPopulaciji;
	}
	
	int Populacija::getStOcenitevDoSedaj() {
		return this->stOcenitevDoSedaj;
	}

	void Populacija::setStGeneracije(int stG) {
		this->stGeneracije = stG;
	}

	void Populacija::setDefaultChromosome(const std::vector<bool> &defaultChromosome) {
		assert((int)defaultChromosome.size()==m);
		assert(osebki);
		for (int i = 0; i < m; i++)
			(*osebki)[0]->biti[i]=defaultChromosome[i];
	}

	void Populacija::resetStGeneracije() {
		this->stGeneracije=0;
		this->stOcenitevDoSedaj=0;
	}

	std::vector<bool>& Populacija::getBestChromosomes() {
		return (*osebki)[0]->biti;
	}

	void Populacija::changeSelectionType(SELECTION sel) {
		if (sel == trenutnaSelekcija) return;
		menjajSelekcijoVNaslednjiGeneraciji=true;
		trenutnaSelekcija=sel;
	}

	const std::vector<Osebek*>* Populacija::getStariOsebki() const {
		return osebkiStaraGeneracija;
	}

	/**
	 *    Vrne kopijo osebkov, ki jih uporabljamo za tekmovanje med selekcijami.
	 * @return 
	 */
	std::vector<Osebek*> *Populacija::getOsebkiCopy() const {
		std::vector<Osebek*> *osebkiCopy = new std::vector<Osebek*>;
		for (int i = 0; i < pop_size; i++) {
			Osebek *o = new Osebek();
			o->biti=(*osebki)[i]->biti;
			osebkiCopy->push_back(o);
		}
		return osebkiCopy;
	}

	void Populacija::setInitOsebki(std::vector<Osebek*> *osebki) {
		assert((int)osebki->size()==pop_size);
		osebki->clear();
		naslednjiIdOsebka=0;
		stGeneracije=0;
		F=0;
		lastF=0;
		ocenaNajboljsegaVPopulaciji=0;
		for (int i = 0; i < pop_size; i++) {
			Osebek *novi=novOsebek(osebki);
			novi->biti=(*osebki)[i]->biti;
		}
	}

	void Populacija::init_pop() {
		//"""Inicializira populacijo tako, da generira naključne posameznike."""
		std::cerr << "Init..." << std::endl;
		stOcenitevDoSedaj=0;
		osebki = new std::vector<Osebek*>;
		naslednjiIdOsebka=0;
		for (int i=0; i < pop_size; i++) {
			Osebek *novi = novOsebek(osebki);
			novi->nakljucnoNapolni();
		}
		for (int i=0; i < pop_size; i++) {
			if ((int)(*osebki)[i]->biti.size()!=m)
				std::cerr << "ni dolžina:"<<(int)(*osebki)[i]->biti.size()<<std::endl;
			//assert((int)(*osebki)[i]->biti.size()==m);
		}
		stGeneracije=0;
		F=0;
		lastF=0;
		ocenaNajboljsegaVPopulaciji=0;
		std::cerr << "Init OK." << std::endl;
	
		ustaviSimulacijo=false;
		simulacijaNaPavzi=false;
	}

	void Populacija::initLog() {
		izpisujSelekcije=true;
		izpisujKrizanja=true;
		izpisujMutacije=true;
	}

	//"""Izpiše podatke o populaciji in njenih osebkih."""
	void Populacija::izpis() {
		std::cerr << "Generacija: " <<  stGeneracije
		<< "@" << ocenaNajboljsegaVPopulaciji << "~" << F << " (€" << stOcenitevDoSedaj << ")" << std::endl;
		assert((int)(*osebki).size()==pop_size);
		for (int i = 0; i < pop_size; i++)
			(*osebki)[i]->izpis();
	}

	Osebek* Populacija::novOsebek(std::vector<Osebek*> *osebki) {
		int sz=(int)(osebki->size());
		assert(sz<pop_size);

		Osebek *novi=new Osebek;
		assert(novi);
		osebki->push_back(novi);

		naslednjiIdOsebka++;
		novi->biti.resize(m);
		novi->m=m;
		novi->populacija=this;
		novi->ocena=-1;
		novi->id=naslednjiIdOsebka;

		return novi;
	}

	void Populacija::zamenjajSelekcijo() {
		switch (trenutnaSelekcija) {
			case SELECTION_PROPORTIONAL:
				izbranaSelekcija = selectionProportional;break;
			case SELECTION_BAKER:
				izbranaSelekcija = selectionBaker;break;
			case SELECTION_LINEAR:
				izbranaSelekcija = selectionLinear;break;
			case SELECTION_NOLINEAR:
				izbranaSelekcija = selectionNolinear;break;
			case SELECTION_TOURNAMENT:
				izbranaSelekcija = selectionTournament;break;
		};
		menjajSelekcijoVNaslednjiGeneraciji=false;
	}

	void Populacija::updateSelectionStats() {
		izbranaSelekcija->shraniBitsInEval((*osebki)[0]->biti, ocenaNajboljsegaVPopulaciji);
	}

	void Populacija::printSelectionStats() {
		std::cerr<<"baker:";
		selectionBaker->printSelectionStats();
		std::cerr<<std::endl<<"linear:";
		selectionLinear->printSelectionStats();
		std::cerr<<std::endl<<"nolinear:";
		selectionNolinear->printSelectionStats();
		std::cerr<<std::endl<<"proportional:";
		selectionProportional->printSelectionStats();
		std::cerr<<std::endl<<"tournament:";
		selectionTournament->printSelectionStats();
		std::cerr<<std::endl;
	}

	//"""Izvede evolucijo populacije v naslednjo generacijo, po pravilih genetske evolucije."""
	bool Populacija::naslednjaGeneracija() {
		if (menjajSelekcijoVNaslednjiGeneraciji) zamenjajSelekcijo();
		std::cerr << "\n-Zacel z obravnavo g=" << stGeneracije<<"-\n";
		oceniVse();
		if (ustaviSimulacijo) return false;
		std::cerr << "\n-Ocenil vse @ g=" << stGeneracije<<"-\n";
		izpis();
		krizajVse();
		std::cerr << "\n-Krizal vse @ g=" << stGeneracije<<"-\n";
		mutirajVseRazenElite();
		std::cerr << "\n-Mutiral vse @ g=" << stGeneracije<<"-\n";
		stGeneracije++;

		bool konec=stGeneracije<G;
		if (konec) updateSelectionStats();
		return konec;
	}

	struct OsebekSort {
		Osebek* instanca;
	};

	struct PrimerjajOsebke {
		//predikat za urejanje
		bool operator()(const OsebekSort &o1, const OsebekSort &o2) {
			return o1.instanca->ocena > o2.instanca->ocena;
		}
	};

	void Populacija::sortirajOsebke() {
		std::vector<OsebekSort> osebkiSort;
		for (int i = 0; i < pop_size; i++) {
			OsebekSort o;
			o.instanca=(*osebki)[i];
			osebkiSort.push_back(o);
		}
		stable_sort(osebkiSort.begin(), osebkiSort.end(), PrimerjajOsebke());
		//sedaj imamo indekse, ki jih bomo zložili
		std::vector<Osebek*> *sortiraniOsebki = new std::vector<Osebek*>;
		for (int i = 0; i < pop_size; i++) {//O(n), OK
			Osebek *osebek = osebkiSort[i].instanca;

			(*sortiraniOsebki).push_back(osebek);
		}
		delete osebki;//starega seznama več ne potrebujemo
		osebki = sortiraniOsebki;
	}

	//"""Oceni vse osebke v populaciji in uredi za izbiro pri selekciji."""
	void Populacija::oceniVse() {
		assert(osebki);
		for (int i = 0; i < pop_size; i++) {
			//to je jaka finta, ampak dobra: je treba res pazit in šparat s sredstvi, če ne simulacija predolgo traja
			if ((*osebki)[i]->ocena<0) {
				stOcenitevDoSedaj++;
				(*osebki)[i]->oceni();
			}
			if (ustaviSimulacijo) return;
			while (simulacijaNaPavzi)
				usleep(100000);
		}
		sortirajOsebke();
		//for (int i = 0; i < pop_size; i++) (*osebki)[i].izpis();
		ocenaNajboljsegaVPopulaciji=(*osebki)[0]->ocena;
		F=0;
		for (int i = 0; i < pop_size; i++) F+=(*osebki)[i]->ocena;
		lastF=F;//pomnim zaradi statistike
	}


	void Populacija::elitizem(std::vector<Osebek*> *noviOsebki) {
		for (int i = 0; i < e; i++) {
			Osebek *osebek = novOsebek(noviOsebki);
			osebek->biti=(*osebki)[i]->biti;
			osebek->ocena=(*osebki)[i]->ocena;
			osebek->id=(*osebki)[i]->id;
		}
	}

	//"""Izvede križanje med osebki in tako generira novo populacijo osebkov, ki predstavljajo novo generacijo."""
	void Populacija::krizajVse() {
		std::vector<Osebek*> *noviOsebki = new std::vector<Osebek*>;
		std::vector<bool> bitiNaj = (*osebki)[0]->biti;

		elitizem(noviOsebki);
		assert((int)noviOsebki->size()==e);
		while ((int)noviOsebki->size()<pop_size) {
			Osebek *oce=0, *mati=0;
			izbranaSelekcija->selekcija(&oce,&mati, osebki);//izberemo očeta in mater

			Osebek *sin=novOsebek(noviOsebki);
			Osebek *hci=0;
			if ((int)noviOsebki->size()<pop_size)//hči izberemo le, če še ni dovolj populacije
				hci=novOsebek(noviOsebki);

			if ((float)rand()/RAND_MAX < p_k) {//rekombinacija?
				int r=1+rand()%(m-2);
				copy(mati->biti.begin(), 	mati->biti.begin()+r, 		sin->biti.begin());
				copy(oce->biti.begin()+r,	oce->biti.end(),			sin->biti.begin()+r);

				if (hci) {
					copy(oce->biti.begin(), 	oce->biti.begin()+r,	hci->biti.begin());
					copy(mati->biti.begin()+r,	mati->biti.end(),		hci->biti.begin()+r);
				}
			} else {
				//naredimo kopije
				for (int i = 0; i < m; i++) sin->biti[i] = oce->biti[i];
				//copy(oce->biti.begin(), 	oce->biti.end(),			sin->biti.begin());
				sin->id = oce->id;
				sin->ocena=oce->ocena;
				if (hci) {
					//copy(mati->biti.begin(), 	mati->biti.end(),		hci->biti.begin());
					for (int i = 0; i < m; i++) hci->biti[i] = mati->biti[i];
					hci->id = mati->id;
					hci->ocena = mati->ocena;
				}

				naslednjiIdOsebka-=2;//ni bilo novih ID-jev v resnici, zato vrednost vrnemo!
			}

			assert((int)sin->biti.size()==m && (!hci ||(int)hci->biti.size()==m));
		}
		
		//pobrišemo staro generacijo, zadnjo še pomnimo za izračune
		std::vector<Osebek*> *brisaniOsebki=osebkiStaraGeneracija;
		osebkiStaraGeneracija=osebki;
		if (brisaniOsebki)
			for (int i = 0; i < pop_size; i++) delete (*brisaniOsebki)[i];
		delete brisaniOsebki;
		osebki=noviOsebki;//nova generacija je zaključena, skopiramo

// 		std::cerr << "Izpis ravno nastalih osebkov, še neocenjenih:"<<std::endl;
// 		izpis();

		Osebek *osebekE = (*osebki)[0];
		if (e!=0) {//preverimo, ali je dobro skopiran vsaj prvi
			for (int i = 0; i < m; i++) {
				assert(osebekE->biti[i] == bitiNaj[i]);
			}
		}
	}

	//"""Mutira vse osebke v populaciji, ki ne spadajo v elito"""
	void Populacija::mutirajVseRazenElite() {
		for (int i = e; i < pop_size; i++)
			(*osebki)[i]->mutiraj(p_m);
	}
}
