#ifndef POPULACIJA_H
#define POPULACIJA_H

#include <vector>

namespace genetic {
	class Populacija;
	class GeneticSimulation;
	class Osebek;
	namespace selection {
		class Selection;
		class SelectionBaker;
		class SelectionLinear;
		class SelectionNolinear;
		class SelectionProportional;
		class SelectionTournament;
	}

	//"""Predstavlja populacijo, ki se skozi evolucijo spreminja."""
	class Populacija {
	public:
		Populacija();
		void init_pop();
		bool naslednjaGeneracija();

		bool setG(int G);
		bool setPop_size(int pop_size);
		void setP_k(float p_k);
		void setP_m(float p_m);
		void setM(int m);
		void setStGeneracije(int stG);
		void setE(int e);
		void setDefaultChromosome(const std::vector<bool> &defaultChromosome);
		void resetStGeneracije();
		std::vector<bool>& getBestChromosomes();
		enum SELECTION {SELECTION_PROPORTIONAL, SELECTION_BAKER, SELECTION_LINEAR,
		                SELECTION_NOLINEAR, SELECTION_TOURNAMENT};
		void changeSelectionType(SELECTION sel);

		int getG();
		int getPop_size();
		float getP_k();
		float getP_m();
		int getM();
		int getStGeneracije();
		int getE();
		float getLastF();
		float getOcenaNajboljsegaVPopulaciji();
		int getStOcenitevDoSedaj();
		const std::vector<Osebek*>* getStariOsebki() const;
		void setInitOsebki(std::vector<Osebek*> *osebki);
		std::vector<Osebek*> *getOsebkiCopy() const;
		void printSelectionStats();

		GeneticSimulation* geneticSimulation;

		bool izpisujSelekcije;
		bool izpisujKrizanja;
		bool izpisujMutacije;
		bool ustaviSimulacijo;
		bool simulacijaNaPavzi;

		//za dostop iz gui
		selection::SelectionBaker *selectionBaker;
		selection::SelectionLinear *selectionLinear;
		selection::SelectionNolinear *selectionNolinear;
		selection::SelectionProportional *selectionProportional;
		selection::SelectionTournament *selectionTournament;

	private:
		void initLog();
		void izpis();
		Osebek* novOsebek(std::vector<Osebek*> *osebki);

		void sortirajOsebke();
		void oceniVse();

		void elitizem(std::vector<Osebek*> *noviOsebki);
		void krizajVse();
		void mutirajVseRazenElite();
		void zamenjajSelekcijo();
		void updateSelectionStats();

		int G;
		int pop_size;
		float p_k;
		float p_m;
		int m;
		int e;
		
		int stGeneracije;
		float F;
		float lastF;//pomnim za prejšnjo generacijo
		float ocenaNajboljsegaVPopulaciji;

		std::vector<Osebek*> *osebki;
		std::vector<Osebek*> *osebkiStaraGeneracija;
		int naslednjiIdOsebka;

		int stOcenitevDoSedaj;
		int trenutnaSelekcija;
		bool menjajSelekcijoVNaslednjiGeneraciji;
		selection::Selection *izbranaSelekcija;
	};
}
#endif
