/***************************************************************************
 *   Copyright (C) 2005 by Aleš Zamuda   *
 *   ales.zamuda@uni-mb.si   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "schemecalculator.h"
#include "osebek.h"

namespace genetic {

	SchemeCalculator::SchemeCalculator(const std::vector<int> &shema,
									   const std::vector<Osebek*> &osebki) : shema(shema), osebki(osebki) {}

	SchemeCalculator::~SchemeCalculator() {}

	double SchemeCalculator::getFPovprecni() {
		return fPovprecni;
	}
	
	double SchemeCalculator::getEvalS0t() {
		return evalS0t;
	}
	
	int SchemeCalculator::getDolzinaSheme() {
		return dolzinaSheme;
	}
	
	int SchemeCalculator::getRedSheme() {
		return redSheme;
	}
	
	int SchemeCalculator::getStPripadajocihS0() {
		return stPripadajocihS0;
	}
	
	double SchemeCalculator::getKolicnikSheme() {
		return kolicnikSheme;
	}
	
	double SchemeCalculator::getNapovedZaShemoS0t1() {
		return napovedZaShemoS0t1;
	}
	
	/**
	 * Izpise vrednost sheme
	 * @param shema - vektor z vrednostmi -1, 1 in 0: -1 pomeni zvezdico, ostali pa enako kar že sta
	 */
	void SchemeCalculator::izracunaj(double F, int pop_size, float p_k, int m, double p_m) {
		redSheme=racunajRedSheme();
		dolzinaSheme=racunajDolzinoSheme();
		stPripadajocihS0=steviloPripadajocih();
		evalS0t=oceneSheme();

		if (stPripadajocihS0>0) evalS0t/=stPripadajocihS0;
		else evalS0t=0;
		fPovprecni = F/pop_size;

		//Izracunamo napoved za shemo:
		kolicnikSheme=evalS0t/(fPovprecni)*
				(1-double(p_k)*double(dolzinaSheme)/double(m-1))*
				(1-double(redSheme)*p_m);
		napovedZaShemoS0t1=double(stPripadajocihS0)*kolicnikSheme;
	}

	/**
	 * Vrne red podane sheme
	 * @param shema - vektor iz 1, 0 in -1 vrednosti za shemo
	 * @return red sheme - število pomembnih vrednosti
	 */
	int SchemeCalculator::racunajRedSheme() {
		int stVseh=0;
		for (int i = 0; i < (int)shema.size(); i++)
			if (shema[i]>=0) stVseh++;
		return stVseh;
	}

	/**
	 * Vrne dolzino podane sheme
	 * @param shema - vektor iz 1, 0 in -1 vrednosti za shemo
	 * @return dolžina sheme - razlika med najbolj oddaljenima pomembnima bitoma
	 */
	int SchemeCalculator::racunajDolzinoSheme() {
		int prvi=0, zadnji=-1;
		for (int i = 0; i < (int)shema.size(); i++)
			if (shema[i]>=0) {
				prvi=i;
				break;
			}

		for (int i = 0; i < (int)shema.size(); i++)
			if (shema[i]>=0)
				zadnji=i;
		return zadnji-prvi+1;
	}

	double SchemeCalculator::oceneSheme() {
		double vsota=0;

		for (int i = 0; i < (int)osebki.size(); i++) {
			Osebek *osebek=osebki[i];
			bool enak=true;
			for (int j = 0; j < (int)shema.size(); j++) {
				if (shema[j]>=0)
					if (shema[j]!=osebek->biti[j]) {
						enak=false;
						break;
					}
			}
			if (enak) vsota+= (double)osebek->ocena;
		}
		return vsota;
	}

	int SchemeCalculator::steviloPripadajocih() {
		int stPripadajocih=0;

		for (int i = 0; i < (int)osebki.size(); i++) {
			Osebek *osebek=osebki[i];
			bool enak=true;
			for (int j = 0; j < (int)shema.size(); j++) {
				if (shema[j]>=0)
					if (shema[j]!=osebek->biti[j]) {
						enak=false;
						break;
					}
			}
			if (enak) stPripadajocih++;
		}
		return stPripadajocih;
	}
}
